import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:pmu_project/components/utils/debounce.dart';
import 'package:pmu_project/data/repositories/api_interface.dart';
import 'package:pmu_project/data/repositories/gravity_falls_repository.dart';
import 'package:pmu_project/domain/models/card_data.dart';
import 'package:pmu_project/presentation/details_page/details_page.dart';
import 'package:pmu_project/presentation/home_page/bloc/bloc.dart';
import 'package:pmu_project/presentation/home_page/bloc/events.dart';
import 'package:pmu_project/presentation/home_page/bloc/state.dart';

part 'card.dart';

class HomePage extends StatefulWidget {
  const HomePage({super.key, required this.title});

  final String title;

  @override
  State<HomePage> createState() => _HomePageState();
}

class _HomePageState extends State<HomePage> {
  final TextEditingController _searchController = TextEditingController();
  final _scrollController = ScrollController();

  @override
  void initState() {
    WidgetsBinding.instance.addPostFrameCallback((_) {
      context.read<HomeBloc>().add(const HomeLoadDataEvent());
    });

    _scrollController.addListener(_onNextPageListener);

    super.initState();
  }

  @override
  void dispose() {
    _searchController.dispose();
    _scrollController.dispose();
    super.dispose();
  }

  void _onNextPageListener() {
    if (_scrollController.offset >=
        _scrollController.position.maxScrollExtent) {
      final bloc = context.read<HomeBloc>();
      if (!bloc.state.isPaginationLoading) {
        bloc.add(HomeLoadDataEvent(
            search: _searchController.text,
            nextPage: bloc.state.data?.nextPage));
      }
    }
  }

  Future<void> _onRefresh() {
    context
        .read<HomeBloc>()
        .add(HomeLoadDataEvent(search: _searchController.text));
    return Future.value(null);
  }

  void _onSearchInputChange(search) {
    Debounce.run(
        action: () =>
            context.read<HomeBloc>().add(HomeLoadDataEvent(search: search)));
  }

  void _showSnackBar(BuildContext context, String text) {
    WidgetsBinding.instance.addPostFrameCallback((_) {
      ScaffoldMessenger.of(context).showSnackBar(SnackBar(
        content: Text(text),
        duration: const Duration(seconds: 2),
        backgroundColor: Colors.black54,
      ));
    });
  }

  void _navigateToDetailsPage(BuildContext context, CardData data) {
    Navigator.push(
      context,
      CupertinoPageRoute(builder: (context) => DetailsPage(data: data)),
    );
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        backgroundColor: Theme.of(context).colorScheme.inversePrimary,
        title: Text(widget.title),
      ),
      body: Column(
        children: [
          Padding(
            padding: const EdgeInsets.only(right: 30, left: 30, top: 20),
            child: CupertinoSearchTextField(
              controller: _searchController,
              onChanged: _onSearchInputChange,
            ),
          ),
          BlocBuilder<HomeBloc, HomeState>(
            builder: (context, state) => state.error != null
                ? Text(
              state.error ?? "",
              style: Theme.of(context)
                  .textTheme
                  .headlineSmall
                  ?.copyWith(color: Colors.red),
            )
                : state.isLoading
                ? const Padding(
              padding: EdgeInsets.only(top: 20),
              child: CircularProgressIndicator(),
            )
                : Expanded(
              child: RefreshIndicator(
                onRefresh: _onRefresh,
                child: ListView.separated(
                  controller: _scrollController,
                  padding: const EdgeInsets.all(30),
                  separatorBuilder: (context, index) =>
                  const SizedBox(height: 20),
                  itemCount: state.data?.data?.length ?? 0,
                  itemBuilder: (context, index) {
                    final data = state.data?.data?[index];

                    return data == null
                        ? const SizedBox.shrink()
                        : _Card.fromData(
                      data,
                      onLike: (String text) =>
                          _showSnackBar(context, text),
                      onTap: () =>
                          _navigateToDetailsPage(context, data),
                    );
                  },
                ),
              ),
            ),
          ),
          BlocBuilder<HomeBloc, HomeState>(
            builder: (context, state) => state.isPaginationLoading
                ? const Padding(padding: EdgeInsets.only(bottom: 10), child: CircularProgressIndicator(),)
                : const SizedBox.shrink(),
          ),
        ],
      ),
    );
  }
}